'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _clamp = require('./clamp');

var _clamp2 = _interopRequireDefault(_clamp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var sum = function sum(a, b) {
  return a + b;
};

var getColumnsWidth = function getColumnsWidth(columns) {
  var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var columnMinWidth = _ref.columnMinWidth;

  return columns.reduce(function (acc, col) {
    var maxWidth = col.maxWidth;
    var defaultWidth = col.defaultWidth;
    var width = col.width;
    var minWidth = col.minWidth;


    if (minWidth === undefined && columnMinWidth != null) {
      minWidth = columnMinWidth;
    }

    if (width === undefined && defaultWidth !== undefined) {
      width = defaultWidth;
    }

    var colWidth = (0, _clamp2.default)(width || 0, minWidth || 0, maxWidth);

    return acc + colWidth;
  }, 0);
};

var getColumnSizeInfo = function getColumnSizeInfo(columns) {
  var _ref2 = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var columnMinWidth = _ref2.columnMinWidth;
  var columnMaxWidth = _ref2.columnMaxWidth;

  var info = columns.reduce(function (acc, col) {
    var defaultWidth = col.defaultWidth;
    var width = col.width;
    var minWidth = col.minWidth;
    var maxWidth = col.maxWidth;


    if (minWidth == null && columnMinWidth != null) {
      minWidth = columnMinWidth;
    }

    if (maxWidth == null && columnMaxWidth != null) {
      maxWidth = columnMaxWidth;
    }

    if (width == null && defaultWidth != null) {
      width = defaultWidth;
    }

    var colWidth = (0, _clamp2.default)(width || 0, minWidth || 0, maxWidth);

    if (width == null) {
      acc.flexCount++;
      if (maxWidth != null) {
        acc.flexWithMaxWidth.push(maxWidth);
      }
    } else {
      acc.width += colWidth;
      acc.sizeCount++;
    }

    acc.minWidth += colWidth;

    return acc;
  }, {
    count: columns.length,
    flexCount: 0,
    flexWithMaxWidth: [],
    sizeCount: 0,
    width: 0,
    minWidth: 0
  });

  if (info.flexCount == info.flexWithMaxWidth.length) {
    info.maxWidth = info.width + info.flexWithMaxWidth.reduce(sum, 0);
  }

  return info;
};

exports.default = getColumnSizeInfo;